/*
 * This file is automatically generated.  DON'T EDIT THIS FILE MANUALLY.
 */

#include "sombok_constants.h"
#include "sombok.h"
#define UNICODE_VERSION "5.1.0"
const char *linebreak_unicode_version = UNICODE_VERSION;

const char *linebreak_propvals_EA[] = {
    "Z", "Na", "N", "A", "W", "H", "F", "ZA", "ZW", NULL
};
const char *linebreak_propvals_GB[] = {
    "CR", "LF", "Control", "Extend", "Prepend", "SpacingMark", "L", "V", 
    "T", "LV", "LVT", "Other", "Virama", "OtherLetter", NULL
};
const char *linebreak_propvals_LB[] = {
    "BK", "CR", "LF", "NL", "SP", "OP", "CL", "CP", "QU", "GL", "NS", "EX", 
    "SY", "IS", "PR", "PO", "NU", "AL", "HL", "ID", "IN", "HY", "BA", "BB", 
    "B2", "ZW", "CM", "WJ", "H2", "H3", "JL", "JV", "JT", "RI", "CB", "SG", 
    "AI", "SA", "XX", NULL
};
const char *linebreak_propvals_SC[] = {
    "Common", "Inherited", "Unknown", "Han", "Hangul", "Thai", "Lao", 
    "Myanmar", "Khmer", "Tai_Le", "New_Tai_Lue", NULL
};

#define M (LINEBREAK_ACTION_MANDATORY)
#define D (LINEBREAK_ACTION_DIRECT)
#define I (LINEBREAK_ACTION_INDIRECT)
#define P (LINEBREAK_ACTION_PROHIBITED)
#define r(cc) static propval_t rule_##cc[]
/* Note: Entries related to BK, CR, CM, LF, NL aren't used by break(). */
    /* BkCrLfNlSpOpClCpQuGlNsExSyIsPrPoNuAlHlIdInHyBaBbB2ZwCmWjH2H3JlJvJtRiCb */
r(BK)={M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M};
r(CR)={M,M,P,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M};
r(LF)={M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M};
r(NL)={M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M,M};
r(SP)={P,P,P,P,P,D,P,D,D,D,D,P,P,P,D,D,D,D,D,D,D,D,D,D,D,P,D,P,D,D,D,D,D,D,D};
r(OP)={P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P,P};
r(CL)={P,P,P,P,P,D,P,D,I,I,P,P,P,P,I,I,D,D,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(CP)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,D,D,D,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(QU)={P,P,P,P,P,P,P,I,I,I,I,P,P,P,I,I,I,I,I,I,I,I,I,I,I,P,I,P,I,I,I,I,I,I,I};
r(GL)={P,P,P,P,P,I,P,I,I,I,I,P,P,P,I,I,I,I,I,I,I,I,I,I,I,P,I,P,I,I,I,I,I,I,I};
r(NS)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,D,D,D,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(EX)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,D,D,D,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(SY)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,D,I,D,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(IS)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,D,I,I,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(PR)={P,P,P,P,P,I,P,D,I,I,I,P,P,P,D,D,I,I,D,I,D,I,I,D,D,P,D,P,I,I,I,I,I,D,D};
r(PO)={P,P,P,P,P,I,P,D,I,I,I,P,P,P,D,D,I,I,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(NU)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,I,I,I,I,D,D,I,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(AL)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,D,I,I,D,D,I,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(HL)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,D,D,D,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(ID)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,I,D,D,D,D,I,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(IN)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,D,D,D,D,D,I,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(HY)={P,P,P,P,P,D,P,D,I,D,I,P,P,P,D,D,I,D,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(BA)={P,P,P,P,P,D,P,D,I,D,I,P,P,P,D,D,D,D,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(BB)={P,P,P,P,P,I,P,I,I,I,I,P,P,P,I,I,I,I,I,I,I,I,I,I,I,P,I,P,I,I,I,I,I,I,D};
r(B2)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,D,D,D,D,D,D,I,I,D,P,P,D,P,D,D,D,D,D,D,D};
r(ZW)={P,P,P,P,P,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,P,D,D,D,D,D,D,D,D,D};
r(CM)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,D,D,D,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(WJ)={P,P,P,P,P,I,P,I,I,I,I,P,P,P,I,I,I,I,I,I,I,I,I,I,I,P,I,P,I,I,I,I,I,I,I};
r(H2)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,I,D,D,D,D,I,I,I,D,D,P,D,P,D,D,D,I,I,D,D};
r(H3)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,I,D,D,D,D,I,I,I,D,D,P,D,P,D,D,D,D,I,D,D};
r(JL)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,I,D,D,D,D,I,I,I,D,D,P,D,P,I,I,I,I,D,D,D};
r(JV)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,I,D,D,D,D,I,I,I,D,D,P,D,P,D,D,D,I,I,D,D};
r(JT)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,I,D,D,D,D,I,I,I,D,D,P,D,P,D,D,D,D,I,D,D};
r(RI)={P,P,P,P,P,D,P,D,I,I,I,P,P,P,D,D,D,D,D,D,D,I,I,D,D,P,D,P,D,D,D,D,D,D,D};
r(CB)={P,P,P,P,P,D,P,D,I,I,D,P,P,P,D,D,D,D,D,D,D,D,D,D,D,P,D,P,D,D,D,D,D,D,D};
#undef r
propval_t *linebreak_rules[] = {
    rule_BK, rule_CR, rule_LF, rule_NL, rule_SP, rule_OP, rule_CL, rule_CP, 
    rule_QU, rule_GL, rule_NS, rule_EX, rule_SY, rule_IS, rule_PR, rule_PO, 
    rule_NU, rule_AL, rule_HL, rule_ID, rule_IN, rule_HY, rule_BA, rule_BB, 
    rule_B2, rule_ZW, rule_CM, rule_WJ, rule_H2, rule_H3, rule_JL, rule_JV, 
    rule_JT, rule_RI, rule_CB
};

size_t linebreak_rulessiz = 35;

unsigned short linebreak_prop_index[] = {
    0, 32, 64, 94, 125, 157, 189, 219, 248, 276, 307, 333, 345, 345, 363,
    345, 345, 345, 392, 343, 345, 345, 420, 345, 452, 452, 469, 498, 529,
    559, 589, 345, 620, 636, 652, 345, 681, 345, 345, 345, 345, 345, 345,
    345, 704, 721, 753, 345, 785, 345, 817, 849, 345, 345, 866, 896, 922,
    940, 961, 345, 345, 987, 1019, 1040, 345, 345, 345, 345, 345, 345, 345,
    345, 1071, 1075, 1106, 1136, 1167, 1171, 1202, 1232, 1071, 1075, 1264,
    1290, 1071, 1075, 1322, 1352, 1167, 1370, 1402, 1432, 1462, 1465, 1497,
    1523, 1554, 1558, 1589, 1432, 1619, 1623, 1655, 1432, 1619, 1465, 1687,
    1717, 1619, 345, 1743, 1775, 1806, 1822, 1854, 345, 1885, 1917, 1949,
    345, 1980, 2012, 345, 2044, 2076, 2101, 2133, 345, 2165, 2186, 2218,
    2249, 2279, 345, 345, 345, 2311, 2311, 2317, 2349, 2349, 2378, 2386,
    2392, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 2418, 2450, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 2482, 345, 345, 2511, 2525,
    2546, 2569, 2569, 2601, 2613, 2643, 1019, 2673, 345, 345, 345, 345, 1455,
    345, 345, 345, 2705, 2733, 2751, 2783, 2805, 2836, 345, 2868, 345, 345,
    345, 345, 345, 345, 345, 2900, 2912, 2942, 2973, 3005, 3036, 345, 345,
    345, 3064, 3096, 3122, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    3154, 3179, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 3211, 3243, 3275, 3306, 3338, 3370, 3402, 940, 3169, 3431,
    3462, 3475, 3507, 3533, 3559, 3585, 3610, 3642, 3673, 3703, 3735, 3765,
    3792, 345, 345, 3805, 3828, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    3860, 3860, 3860, 3860, 3882, 3860, 3860, 3914, 3930, 3962, 3994, 4022,
    4052, 4079, 345, 4107, 4139, 345, 345, 345, 345, 345, 4155, 4185, 4216,
    4248, 345, 4275, 4301, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 4330, 345, 4355, 4383, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 4413, 345, 345, 345, 345, 345, 345, 345, 3154, 4445, 4477, 345, 345,
    4509, 4536, 4536, 4548, 4536, 4536, 4536, 4536, 4536, 4536, 4546, 4568,
    4600, 4632, 4664, 4693, 4722, 4751, 4693, 4780, 4812, 4830, 4536, 4536,
    4520, 4544, 4536, 4858, 4537, 4536, 4890, 4536, 4536, 4536, 4536, 4537,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 345, 345,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4675, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4909, 4536, 4934, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 4953, 1019, 345, 4970, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 5000, 5029, 345, 5041,
    1556, 5065, 5093, 345, 1019, 5119, 5144, 345, 345, 345, 345, 345, 345,
    5167, 5196, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240,
    5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232,
    5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252,
    5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244,
    5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236,
    5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228,
    5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248,
    5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240,
    5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232,
    5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252,
    5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244,
    5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236,
    5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228,
    5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248,
    5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240,
    5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232,
    5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252,
    5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244,
    5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236,
    5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228,
    5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248,
    5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240,
    5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232,
    5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252,
    5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244,
    5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236,
    5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228,
    5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240, 5244, 5248,
    5252, 5228, 5232, 5236, 5240, 5244, 5248, 5252, 5228, 5232, 5236, 5240,
    5244, 5248, 5280, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636, 636,
    636, 636, 636, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536, 4536,
    4536, 4536, 4536, 4536, 4536, 4536, 4536, 2419, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 5284, 345, 345,
    345, 345, 345, 5316, 5348, 5380, 5408, 5439, 345, 345, 345, 5451, 5483,
    5515, 5545, 5577, 5594, 5626, 5657, 5689, 345, 345, 345, 345, 345, 345,
    345, 345, 5721, 345, 345, 345, 345, 345, 345, 5724, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 5754, 345, 2466, 345, 345, 345,
    345, 345, 345, 1019, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 5754, 345, 345, 345,
    345, 345, 345, 345, 5786, 5802, 5834, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 5858, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 5885, 5914, 5936, 345, 345, 345, 345, 5966, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 5984, 5998,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345,
    345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345, 345
};

propval_t linebreak_prop_array[] = {
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_BA, EA_Z, GB_Control, SC_Unknown,
    LB_LF, EA_Z, GB_LF, SC_Unknown, LB_BK, EA_Z, GB_Control, SC_Unknown,
    LB_BK, EA_Z, GB_Control, SC_Unknown, LB_CR, EA_Z, GB_CR, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_SP, EA_Na, GB_Other, SC_Unknown, LB_EX, EA_Na, GB_Other, SC_Unknown,
    LB_QU, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_PR, EA_Na, GB_Other, SC_Unknown, LB_PO, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_QU, EA_Na, GB_Other, SC_Unknown,
    LB_OP, EA_Na, GB_Other, SC_Unknown, LB_CL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_PR, EA_Na, GB_Other, SC_Unknown,
    LB_IS, EA_Na, GB_Other, SC_Unknown, LB_HY, EA_Na, GB_Other, SC_Unknown,
    LB_IS, EA_Na, GB_Other, SC_Unknown, LB_SY, EA_Na, GB_Other, SC_Unknown,
    LB_NU, EA_Na, GB_Other, SC_Unknown, LB_NU, EA_Na, GB_Other, SC_Unknown,
    LB_NU, EA_Na, GB_Other, SC_Unknown, LB_NU, EA_Na, GB_Other, SC_Unknown,
    LB_NU, EA_Na, GB_Other, SC_Unknown, LB_NU, EA_Na, GB_Other, SC_Unknown,
    LB_NU, EA_Na, GB_Other, SC_Unknown, LB_NU, EA_Na, GB_Other, SC_Unknown,
    LB_NU, EA_Na, GB_Other, SC_Unknown, LB_NU, EA_Na, GB_Other, SC_Unknown,
    LB_IS, EA_Na, GB_Other, SC_Unknown, LB_IS, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_EX, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_OP, EA_Na, GB_Other, SC_Unknown,
    LB_PR, EA_Na, GB_Other, SC_Unknown, LB_CL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_OP, EA_Na, GB_Other, SC_Unknown,
    LB_BA, EA_Na, GB_Other, SC_Unknown, LB_CL, EA_Na, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_NL, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Control, SC_Unknown, LB_GL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_A, GB_Other, SC_Unknown, LB_PO, EA_Na, GB_Other, SC_Unknown,
    LB_PR, EA_Na, GB_Other, SC_Unknown, LB_PR, EA_A, GB_Other, SC_Unknown,
    LB_PR, EA_Na, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_QU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_Na, GB_Other, SC_Unknown,
    LB_BA, EA_ZA, GB_Control, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_Na, GB_Other, SC_Unknown, LB_PO, EA_A, GB_Other, SC_Unknown,
    LB_PR, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_BB, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_QU, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_OP, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_BB, EA_A, GB_Other, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_GL, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_GL, EA_ZA, GB_Extend, SC_Unknown,
    LB_GL, EA_ZA, GB_Extend, SC_Unknown, LB_GL, EA_ZA, GB_Extend, SC_Unknown,
    LB_GL, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_IS, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_IS, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown,
    LB_AL, EA_Z, GB_Control, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown,
    LB_AL, EA_Z, GB_Control, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_PO, EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown,
    LB_PO, EA_N, GB_Other, SC_Unknown, LB_IS, EA_N, GB_Other, SC_Unknown,
    LB_IS, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_IS, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other,
    SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_PR, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Virama,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Virama,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Virama,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N,
    GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other,
    SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai,
    LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA,
    EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N,
    GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other,
    SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai,
    LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA,
    EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N,
    GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other,
    SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai,
    LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA,
    EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N,
    GB_Other, SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_N, GB_Other,
    SC_Thai, LB_SA, EA_N, GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend,
    SC_Thai, LB_SA, EA_N, GB_Extend, SC_Thai, LB_SA, EA_N, GB_Extend,
    SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend,
    SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend,
    SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend,
    SC_Thai, LB_SA, EA_Z, GB_Virama, SC_Thai, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other,
    SC_Unknown, LB_SA, EA_N, GB_Prepend, SC_Thai, LB_SA, EA_N, GB_Prepend,
    SC_Thai, LB_SA, EA_N, GB_Prepend, SC_Thai, LB_SA, EA_N, GB_Prepend,
    SC_Thai, LB_SA, EA_N, GB_Prepend, SC_Thai, LB_SA, EA_N, GB_Extend,
    SC_Thai, LB_SA, EA_N, GB_Other, SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai,
    LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA,
    EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z,
    GB_Extend, SC_Thai, LB_SA, EA_Z, GB_Extend, SC_Thai, LB_SA, EA_Z,
    GB_Extend, SC_Thai, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N,
    GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N,
    GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N,
    GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other,
    SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA,
    EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N,
    GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other,
    SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao,
    LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N,
    GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao,
    LB_SA, EA_N, GB_Other, SC_Lao, LB_SA, EA_N, GB_Extend, SC_Lao, LB_SA,
    EA_Z, GB_Extend, SC_Lao, LB_SA, EA_N, GB_Extend, SC_Lao, LB_SA, EA_N,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_N, GB_Other,
    SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_SA, EA_N, GB_Prepend, SC_Lao, LB_SA, EA_N, GB_Prepend,
    SC_Lao, LB_SA, EA_N, GB_Prepend, SC_Lao, LB_SA, EA_N, GB_Prepend, SC_Lao,
    LB_SA, EA_N, GB_Prepend, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA,
    EA_Z, GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_SA, EA_Z,
    GB_Extend, SC_Lao, LB_SA, EA_Z, GB_Extend, SC_Lao, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_Lao, LB_SA, EA_N, GB_Other, SC_Lao, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown,
    LB_GL, EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown,
    LB_BB, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown,
    LB_GL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown,
    LB_GL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_EX, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_BA, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_BA, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BB,
    EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark,
    SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend,
    SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend,
    SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_Z,
    GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z,
    GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z,
    GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N,
    GB_SpacingMark, SC_Myanmar, LB_SA, EA_Z, GB_Virama, SC_Myanmar, LB_SA,
    EA_Z, GB_Virama, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar,
    LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_Z, GB_Extend,
    SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N, GB_Other,
    SC_Myanmar, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other,
    SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other,
    SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other,
    SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_N,
    GB_SpacingMark, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA,
    EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA,
    EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA,
    EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark,
    SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_N,
    GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N,
    GB_SpacingMark, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar,
    LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark,
    SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_N,
    GB_SpacingMark, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar,
    LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar,
    LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark,
    SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_Z,
    GB_Extend, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA, EA_N,
    GB_SpacingMark, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar,
    LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark,
    SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar, LB_SA, EA_N,
    GB_SpacingMark, SC_Myanmar, LB_SA, EA_Z, GB_Extend, SC_Myanmar, LB_SA,
    EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_SpacingMark, SC_Myanmar,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_SA, EA_N, GB_Other, SC_Myanmar, LB_SA, EA_N, GB_Other, SC_Myanmar,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L,
    SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown,
    LB_JL, EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JL,
    EA_W, GB_L, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_JL, EA_W, GB_L, SC_Unknown, LB_JV, EA_N, GB_V,
    SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown,
    LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV,
    EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V,
    SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown,
    LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV,
    EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V,
    SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown,
    LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV,
    EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V,
    SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown,
    LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV,
    EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V,
    SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown, LB_JV, EA_N, GB_V, SC_Unknown,
    LB_JV, EA_N, GB_V, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_JT,
    EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T,
    SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown,
    LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT,
    EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T,
    SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown,
    LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT,
    EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T,
    SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown,
    LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT,
    EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T,
    SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown,
    LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT,
    EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_JT, EA_N, GB_T,
    SC_Unknown, LB_JT, EA_N, GB_T, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Virama,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_N, GB_Other, SC_Khmer, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_Z, GB_Control, SC_Khmer, LB_SA, EA_Z, GB_Control,
    SC_Khmer, LB_SA, EA_N, GB_SpacingMark, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_N, GB_SpacingMark, SC_Khmer, LB_SA, EA_N,
    GB_SpacingMark, SC_Khmer, LB_SA, EA_N, GB_SpacingMark, SC_Khmer, LB_SA,
    EA_N, GB_SpacingMark, SC_Khmer, LB_SA, EA_N, GB_SpacingMark, SC_Khmer,
    LB_SA, EA_N, GB_SpacingMark, SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer,
    LB_SA, EA_N, GB_SpacingMark, SC_Khmer, LB_SA, EA_N, GB_SpacingMark,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Extend,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_SA, EA_Z, GB_Virama,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_NS, EA_N, GB_Other,
    SC_Unknown, LB_SA, EA_N, GB_Other, SC_Khmer, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_Khmer, LB_SA, EA_Z, GB_Extend, SC_Khmer, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other,
    SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other,
    SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_GL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other,
    SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_SA, EA_N, GB_Other, SC_Tai_Le, LB_SA, EA_N, GB_Other,
    SC_Tai_Le, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA,
    EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue,
    LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA,
    EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue,
    LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA,
    EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue,
    LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA,
    EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue,
    LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA,
    EA_N, GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_SpacingMark, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue,
    LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_SA,
    EA_N, GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark,
    SC_New_Tai_Lue, LB_SA, EA_N, GB_SpacingMark, SC_New_Tai_Lue, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N,
    GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_SA, EA_N,
    GB_Other, SC_New_Tai_Lue, LB_SA, EA_N, GB_Other, SC_New_Tai_Lue, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown,
    LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_Virama, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_GL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_ZW, EA_Z, GB_Control, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_BA,
    EA_A, GB_Other, SC_Unknown, LB_GL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_A, GB_Other, SC_Unknown, LB_B2,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_A, GB_Other, SC_Unknown, LB_QU, EA_A, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_A, GB_Other, SC_Unknown, LB_QU, EA_A, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_IN,
    EA_A, GB_Other, SC_Unknown, LB_IN, EA_A, GB_Other, SC_Unknown, LB_IN,
    EA_A, GB_Other, SC_Unknown, LB_BA, EA_A, GB_Other, SC_Unknown, LB_BK,
    EA_Z, GB_Control, SC_Unknown, LB_BK, EA_Z, GB_Control, SC_Unknown, LB_CM,
    EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM,
    EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM,
    EA_Z, GB_Control, SC_Unknown, LB_GL, EA_N, GB_Other, SC_Unknown, LB_PO,
    EA_A, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown, LB_PO,
    EA_A, GB_Other, SC_Unknown, LB_PO, EA_A, GB_Other, SC_Unknown, LB_PO,
    EA_N, GB_Other, SC_Unknown, LB_PO, EA_A, GB_Other, SC_Unknown, LB_PO,
    EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_NS,
    EA_N, GB_Other, SC_Unknown, LB_NS, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_IS, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_NS,
    EA_N, GB_Other, SC_Unknown, LB_NS, EA_N, GB_Other, SC_Unknown, LB_NS,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_WJ, EA_Z, GB_Control, SC_Unknown, LB_AL,
    EA_Z, GB_Control, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown, LB_AL,
    EA_Z, GB_Control, SC_Unknown, LB_AL, EA_Z, GB_Control, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM,
    EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM,
    EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM,
    EA_Z, GB_Control, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_PO,
    EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_H, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_N, GB_Other, SC_Unknown, LB_PR, EA_A, GB_Other, SC_Unknown, LB_PR,
    EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_N, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_PO, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_PO, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_PR, EA_N, GB_Other, SC_Unknown, LB_PR,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_A, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_EX,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other, SC_Unknown, LB_AI,
    EA_A, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_N, GB_Other, SC_Unknown, LB_AI, EA_N, GB_Other, SC_Unknown, LB_AI,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other, SC_Unknown, LB_CL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_Na, GB_Other, SC_Unknown, LB_CL, EA_Na, GB_Other, SC_Unknown, LB_OP,
    EA_Na, GB_Other, SC_Unknown, LB_CL, EA_Na, GB_Other, SC_Unknown, LB_OP,
    EA_Na, GB_Other, SC_Unknown, LB_CL, EA_Na, GB_Other, SC_Unknown, LB_OP,
    EA_Na, GB_Other, SC_Unknown, LB_CL, EA_Na, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_Na, GB_Other, SC_Unknown, LB_CL, EA_Na, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_QU,
    EA_N, GB_Other, SC_Unknown, LB_QU, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_OP,
    EA_N, GB_Other, SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_EX,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_OP,
    EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_CL,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_CM, EA_ZW, GB_Extend, SC_Unknown, LB_CM,
    EA_ZW, GB_Extend, SC_Unknown, LB_CM, EA_ZW, GB_Extend, SC_Unknown, LB_CM,
    EA_ZW, GB_Extend, SC_Unknown, LB_CM, EA_ZW, GB_Extend, SC_Unknown, LB_CM,
    EA_ZW, GB_Extend, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_ZW, GB_Extend, SC_Unknown, LB_CM, EA_ZW, GB_Extend, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_NS,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID,
    EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_EX,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_BB, EA_N, GB_Other, SC_Unknown, LB_BB,
    EA_N, GB_Other, SC_Unknown, LB_EX, EA_N, GB_Other, SC_Unknown, LB_EX,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown,
    LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Virama,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other,
    SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_N,
    GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM,
    EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown,
    LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA,
    EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_H2,
    EA_W, GB_LV, SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W,
    GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H2, EA_W, GB_LV,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_H3, EA_W, GB_LVT,
    SC_Unknown, LB_H3, EA_W, GB_LVT, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_OP, EA_N, GB_Other,
    SC_Unknown, LB_CL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_PO, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_ZA, GB_Extend,
    SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend,
    SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend,
    SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend,
    SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend,
    SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend,
    SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend,
    SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_CM, EA_ZA, GB_Extend,
    SC_Unknown, LB_CM, EA_ZA, GB_Extend, SC_Unknown, LB_IS, EA_W, GB_Other,
    SC_Unknown, LB_CL, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_IS, EA_W, GB_Other, SC_Unknown, LB_IS, EA_W, GB_Other,
    SC_Unknown, LB_EX, EA_W, GB_Other, SC_Unknown, LB_EX, EA_W, GB_Other,
    SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_IN, EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NS, EA_W, GB_Other,
    SC_Unknown, LB_NS, EA_W, GB_Other, SC_Unknown, LB_EX, EA_W, GB_Other,
    SC_Unknown, LB_EX, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_OP, EA_W, GB_Other, SC_Unknown, LB_CL, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_ID, EA_W, GB_Other, SC_Unknown, LB_PR, EA_W, GB_Other,
    SC_Unknown, LB_PO, EA_W, GB_Other, SC_Unknown, LB_ID, EA_W, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_WJ, EA_Z, GB_Control,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_EX, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_PR, EA_F, GB_Other, SC_Unknown, LB_PO, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_OP, EA_F, GB_Other, SC_Unknown, LB_CL, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_CL, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_CL, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_NS, EA_F, GB_Other, SC_Unknown, LB_NS, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_EX, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_OP, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_CL, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_OP, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_CL, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_OP, EA_F, GB_Other,
    SC_Unknown, LB_CL, EA_F, GB_Other, SC_Unknown, LB_CL, EA_H, GB_Other,
    SC_Unknown, LB_OP, EA_H, GB_Other, SC_Unknown, LB_CL, EA_H, GB_Other,
    SC_Unknown, LB_CL, EA_H, GB_Other, SC_Unknown, LB_NS, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_NS, EA_H, GB_Other,
    SC_Unknown, LB_NS, EA_H, GB_Other, SC_Unknown, LB_NS, EA_H, GB_Other,
    SC_Unknown, LB_NS, EA_H, GB_Other, SC_Unknown, LB_NS, EA_H, GB_Other,
    SC_Unknown, LB_NS, EA_H, GB_Other, SC_Unknown, LB_NS, EA_H, GB_Other,
    SC_Unknown, LB_NS, EA_H, GB_Other, SC_Unknown, LB_NS, EA_H, GB_Other,
    SC_Unknown, LB_NS, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_NS, EA_H, GB_Extend,
    SC_Unknown, LB_NS, EA_H, GB_Extend, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_PO, EA_F, GB_Other, SC_Unknown, LB_PR, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_ID, EA_F, GB_Other,
    SC_Unknown, LB_ID, EA_F, GB_Other, SC_Unknown, LB_PR, EA_F, GB_Other,
    SC_Unknown, LB_PR, EA_F, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_H, GB_Other,
    SC_Unknown, LB_AL, EA_H, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Control,
    SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control,
    SC_Unknown, LB_CB, EA_N, GB_Other, SC_Unknown, LB_AI, EA_A, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_CM, EA_Z, GB_Virama, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_BA, EA_N, GB_Other,
    SC_Unknown, LB_BA, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other,
    SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N, GB_Extend,
    SC_Unknown, LB_CM, EA_N, GB_SpacingMark, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z,
    GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL, EA_N,
    GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM, EA_N,
    GB_SpacingMark, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM,
    EA_N, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM,
    EA_N, GB_Extend, SC_Unknown, LB_CM, EA_N, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM,
    EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM,
    EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM,
    EA_Z, GB_Control, SC_Unknown, LB_CM, EA_Z, GB_Control, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_CM, EA_Z, GB_Extend, SC_Unknown, LB_CM,
    EA_Z, GB_Extend, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_AL,
    EA_N, GB_Other, SC_Unknown, LB_AL, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown, LB_NU,
    EA_N, GB_Other, SC_Unknown, LB_NU, EA_N, GB_Other, SC_Unknown
};

